[v-cloak] > * {
  display: none;
}

[v-cloak]::before {
  content: "Loading…";
}

.v-content {
  transition: none !important;
}

.v-application .director {
  background-color: #3c4652 !important;
  border-color: #3c4652 !important;
}

#network {
  width: 100%;
  height: 470px;
  border-right: 1px solid rgba(0, 0, 0, 0.12);
}

.pointer {
  cursor: pointer;
}

.graph .details {
  font-weight: 300;
  font-size: 14px;
}

.graph .node rect {
  stroke-width: 1.5px;
  stroke: #bbb;
  fill: #fff;
}

.graph .status {
  height: 50px;
  width: 10px;
  display: block;
  float: left;
  border-top-left-radius: 5px;
  border-bottom-left-radius: 5px;
  margin-right: 8px;
}

.graph .success .status {
  background-color: #4caf50;
}

.graph .error .status {
  background-color: #f44336;
}

.graph .canceled .status {
  background-color: #b71c1c;
}

.graph .pending .status {
  background-color: #787777;
}

.graph .progress .status {
  background-color: #2196f3;
}

.graph .name {
  margin-top: 10px;
}

.graph .node g div {
  width: 200px;
  height: 50px;
  color: #333;
}

.graph .edgeLabel text {
  width: 50px;
  fill: #fff;
}

.graph .edgePath path {
  stroke: #999;
  stroke-width: 1.5px;
  fill: #999;
}

.v-application .overline {
  font-size: 0.75rem !important;
}

.v-application code::before {
  content: "";
}

.v-application code {
  white-space: pre-wrap;
  overflow-x: auto;
  padding: 10px;
  display: block !important;
  white-space: pre;
  background-color: #f5f5f5;
  box-shadow: none;
  font-weight: normal;
  font-size: 90%;
}

.monospacedPayload textarea {
  font-family: "Monospace";
}
